package com.atguigu.tingshu.album.service;

import com.jd.platform.hotkey.client.callback.JdHotKeyStore;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;


/**
 * @version 1.0
 */
@Slf4j
@Component
public class CacheService {
    @Resource
    private RedisTemplate<String, String> redisTemplate;


    public Object getFromRedis(String key) {
        return redisTemplate.opsForValue().get(key);
    }


    /**
     * Object getValue(String key)，该方法是一个整合方法，相当于isHotKey和get两个方法的整合，该方法直接返回本地缓存的value。
     * 如果是热key，则存在两种情况，
     *      1是返回value 说明是一个热点key,已经加入到本地缓存，从本地缓存获取
     *      2是返回null。如果不是热key，从Redis中获取即可，并且将key上报到探测集群进行数量探测。
     * @param key
     * @return
     */
    public Object get(String key) {
        Object localCache = JdHotKeyStore.getValue(key);
        //如果已经缓存过了
        if (localCache != null) {
            System.out.println("is hot key");
            return localCache;
        } else {
            Object redisCache = getFromRedis(key);
            JdHotKeyStore.smartSet(key, redisCache);
            return redisCache;
        }
    }

    public void set(String key, String value) {
        redisTemplate.opsForValue().set(key, value);
    }

    public void remove(String key) {
        JdHotKeyStore.remove(key);
    }

}
