package com.atguigu.tingshu.dispatch.job;

import com.atguigu.tingshu.search.client.SearchFeignClient;
import com.atguigu.tingshu.user.client.UserFeignClient;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class DispatchHandler {

    @Autowired
    private SearchFeignClient searchFeignClient;

    @Autowired
    private UserFeignClient userFeignClient;

    /**
     * 更新Redis中小时榜任务
     */
    @XxlJob("updateLatelyAlbumRanking")
    public void updateLatelyAlbumRanking() {
        log.info("更新Redis中小时榜任务.");
        searchFeignClient.updateLatelyAlbumRanking();
    }


    /**
     * 更新VIP会员标识
     */
    @XxlJob("updateVipExpireStatus")
    public void updateVipExpireStatus() {
        log.info("更新VIP会员标识任务.");
        userFeignClient.updateVipExpireStatus();
    }


    /**
     * 测试任务:任务逻辑中需要有些参数动态传入在调度中心管理页面动态设置
     * 需求：银行贷款客户30W，在还款日21号前几天（17-20）给客户发邮件通知，邮件内容：或者发送短信通知。内容：客户还款日，请及时还款，如果客户还款了，则发送短信通知客户，短信内容：客户还款了，请及时查看账单，如果客户没有还款，则发送短信通知客户，短信内容：客户没有还款，请及时查看账单，如果客户没有还款，则发送短信通知客户，短信内容：客户没有还款，请及时查看账单，如果客户没有还款，则发送短信通知客户
     * 场景：大量任务需要并发执行，如何提升任务的执行效率？ 解决：采用分片广播，一次任务调度调度所有分片全部同时执行，提高任务的执行效率。
     * 问题：如何避免任务重复执行 解决：使用分片参数开发
     * 每隔5秒触发一次广播任务：某个节点执行任务查询条件：任务状态：未执行 and 用户ID % 分片总数 = 分片序号 limit 100;
     */
    @XxlJob("testJob")
    public void testJob() {
        String jobParam = XxlJobHelper.getJobParam();
        long jobId = XxlJobHelper.getJobId();
        log.info("测试任务，任务ID：{}，任务参数：{}", jobId, jobParam);

        int shardIndex = XxlJobHelper.getShardIndex();
        int shardTotal = XxlJobHelper.getShardTotal();
        log.info("测试任务，分片参数：当前分片序号={}，总分片数={}", shardIndex, shardTotal);
    }
}
