package com.atguigu.tingshu.order.service;

import com.atguigu.tingshu.model.order.OrderInfo;
import com.atguigu.tingshu.vo.order.OrderInfoVo;
import com.atguigu.tingshu.vo.order.TradeVo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;

public interface OrderInfoService extends IService<OrderInfo> {


    /**
     * 对购买商品（VIP会员、专辑、声音）封装订单结算页所需要数据
     * @param userId 用户ID
     * @param tradeVo {购买商品类别,商品ID,声音数量}
     * @return
     */
    OrderInfoVo trade(Long userId, TradeVo tradeVo);

    /**
     * 提交订单
     * @param orderInfoVo
     * @return 订单编号
     */
    String submitOrder(Long userId, OrderInfoVo orderInfoVo);

    /**
     * 保存订单及相关信息
     * @param orderInfoVo
     * @return
     */
    OrderInfo saveOrderInfo(OrderInfoVo orderInfoVo);

    /**
     * 根据订单编号查询订单信息(包含订单明细，优惠列表)
     * @param orderNo
     * @return
     */
    OrderInfo getOrderInfo(String orderNo);

    /**
     * 分页查询当前用户订单列表
     * @param pageInfo
     * @param userId
     * @return
     */
    Page<OrderInfo> findUserOrderPage(Page<OrderInfo> pageInfo, Long userId);

    /**
     * 根据订单ID检查订单支付状态，如果未支付则更新订单状态为已取消
     * @param orderId
     */
    void cancelOrder(Long orderId);

    /**
     * 用户付款成功后更新订单状态以及虚拟物品发货
     *
     * @param orderNo
     * @return
     */
    void orderPaySuccess(String orderNo);
}
